//--------------------------------------------------------------
/*

  XpressNetMaster Interface for Arduino
  
Funktionsumfang:  
- Fahren per LokMaus2 und MultiMaus
- Schalten von DCC Weichen mit der MultiMaus (not tested 15.04.15)
  
*/
#if defined(XPRESSNET)

XpressNetMasterClass XpressNet;

//Pin Config:
#define XNetTxRxPin  9            //Port for Send/Receive on MAX485
#define XNetFahrstufen Loco128    //Standard Fahrstufen Setup

byte XNetUserOps = 0x00;
boolean XNetReturnLocoInfo = false;
boolean XNetReturnLocoFkt = false;

//--------------------------------------------------------------
//Change Power Status
void notifyXNetPower(uint8_t State) {
  #if defined(DEBUG)
  Serial.print("XNetPower: ");
  Serial.println(State, HEX);
  #endif
  setPower(State);
}

//--------------------------------------------------------------
void notifyXNetgiveLocoInfo(uint8_t UserOps, uint16_t Address) {
  XNetReturnLocoInfo = true;
  XNetUserOps = UserOps;
  dcc.getLocoStateFull(Address, false);
}

//--------------------------------------------------------------
void notifyXNetgiveLocoFunc(uint8_t UserOps, uint16_t Address) {
  XNetReturnLocoFkt = true;
  XNetUserOps = UserOps;
  dcc.getLocoStateFull(Address, false);
}

//--------------------------------------------------------------
void notifyXNetLocoDrive14(uint16_t Address, uint8_t Speed) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", S14:");
  Serial.println(Speed, BIN);
  #endif
  if (Speed == 0) 
    dcc.setSpeed14(Address, (dcc.getLocoDir(Address) << 7) | (Speed & B01111111));
  else dcc.setSpeed14(Address, Speed);
}

//--------------------------------------------------------------
void notifyXNetLocoDrive28(uint16_t Address, uint8_t Speed) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", S28:");
  Serial.println(Speed, BIN);
  #endif
  if (Speed == 0)
    dcc.setSpeed28(Address, (dcc.getLocoDir(Address) << 7) | (Speed & B01111111));
  else dcc.setSpeed28(Address, Speed);
}

//--------------------------------------------------------------
void notifyXNetLocoDrive128(uint16_t Address, uint8_t Speed) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", S128:");
  Serial.println(Speed, BIN);
  #endif
  if (Speed == 0) 
    dcc.setSpeed128(Address, (dcc.getLocoDir(Address) << 7) | (Speed & B01111111));
  else dcc.setSpeed128(Address, Speed);
}

//--------------------------------------------------------------
void notifyXNetLocoFunc1(uint16_t Address, uint8_t Func1) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", F1:");
  Serial.println(Func1, BIN);
  #endif
  dcc.setFunctions0to4(Address, Func1);	//- F0 F4 F3 F2 F1
}

//--------------------------------------------------------------
void notifyXNetLocoFunc2(uint16_t Address, uint8_t Func2) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", F2:");
  Serial.println(Func2, BIN);
  #endif
  dcc.setFunctions5to8(Address, Func2);	//- F8 F7 F6 F5
}

//--------------------------------------------------------------
void notifyXNetLocoFunc3(uint16_t Address, uint8_t Func3) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", F3:");
  Serial.println(Func3, BIN);
  #endif
  dcc.setFunctions9to12(Address, Func3);	//- F12 F11 F10 F9
}

//--------------------------------------------------------------
void notifyXNetLocoFunc4(uint16_t Address, uint8_t Func4) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", F4:");
  Serial.println(Func4, BIN);
  #endif
  dcc.setFunctions13to20(Address, Func4);	//F20 F19 F18 F17 F16 F15 F14 F13
}

//--------------------------------------------------------------
void notifyXNetLocoFunc5(uint16_t Address, uint8_t Func5) {
  #if defined(DEBUG)
  Serial.print("XNet A:");
  Serial.print(Address);
  Serial.print(", F5:");
  Serial.println(Func5, BIN);
  #endif
  dcc.setFunctions21to28(Address, Func5);	//F28 F27 F26 F25 F24 F23 F22 F21
}

//--------------------------------------------------------------
void notifyXNetTrntInfo(uint8_t UserOps, uint8_t Address, uint8_t data) {
  int adr = ((Address * 4) + ((data & 0x01) * 2));
  byte pos = data << 4;
  bitWrite(pos, 7, 1);  //command completed!
  if (dcc.getBasicAccessoryInfo(adr) == false)
    bitWrite(pos, 0, 1);
  else bitWrite(pos, 1, 1);  
  if (dcc.getBasicAccessoryInfo(adr+1) == false)
    bitWrite(pos, 2, 1);  
  else bitWrite(pos, 3, 1);    
  XpressNet.SetTrntStatus(UserOps, Address, pos);
}

//--------------------------------------------------------------
void notifyXNetTrnt(uint16_t Address, uint8_t data) {
  if (bitRead(data,3) == 0x01) {  //Weiche Aktiv == HIGH?
    #if defined(DEBUG)
    Serial.print("XNet TA:");
    Serial.print(Address);
    Serial.print(", P:");
    Serial.println(data, BIN);
    #endif
    dcc.setBasicAccessoryPos(Address,data & 0x01);
  }
}

//--------------------------------------------------------------
void notifyXNetDirectCV(uint8_t CV, uint8_t data) {
  dcc.opsProgDirectCV(CV,data); 
}

#endif
